program project1;

uses Classes, StrUtils;

const
  FORMAT_HEX = 0;
  FORMAT_OCT = 1;
  FORMAT_BIN = 2;

type

  TApiData = LongWord;

  (* This API expects a LongWord made up from
     1 Byte (Bits 24..31): A Digit Count (Size)
     1 Byte (Bits 16..23): A Format
     1 Word (Bits  0..15): A Number
  *)

  { TMyAPIBase }

  TMyAPIBase = class
  private
    FList: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(ASize: Byte; AFormat: Byte; ANumber: Word);
    function GetText: String;
  end;

  TMyAPI = class(TMyAPIBase)
  type
    TApiDataStruct = packed record
      Size: Byte;
      Format: Byte;
      Number: Word;
    end;
  public
    procedure ApiStore(AData: TApiData);
    procedure Print;
  end;

constructor TMyAPIBase.Create;
begin
  inherited Create;
  FList := TStringList.Create;
end;

destructor TMyAPIBase.Destroy;
begin
  FList.Destroy;
  inherited Destroy;
end;

procedure TMyAPIBase.Add(ASize: Byte; AFormat: Byte; ANumber: Word);
begin
  case AFormat of
    FORMAT_HEX:
      FList.Add(Dec2Numb(ANumber, ASize, 16));
    FORMAT_OCT:
      FList.Add(Dec2Numb(ANumber, ASize, 8));
    FORMAT_BIN:
      FList.Add(Dec2Numb(ANumber, ASize, 2));
    else
      FList.Add(Dec2Numb(ANumber, ASize, AFormat));
  end;
end;

function TMyAPIBase.GetText: String;
begin
  Result := FList.Text;
end;

procedure TMyAPI.ApiStore(AData: TApiData);
var
  d: TApiDataStruct;
begin
  d := TApiDataStruct(AData);
  with d do
    Add(Size, Format, Number);
end;

procedure TMyAPI.Print;
begin
  WriteLn(GetText);
end;

function GetApiValue(S, F, N: Integer): TApiData;
begin
  Result := S << 24 + F << 16 + N;
end;

var
  Api: TMyAPI;
  Val: TApiData;

begin
  Api := TMyAPI.Create;

  Val := GetApiValue(4, FORMAT_HEX, 42);
  Api.ApiStore(Val);

  Val := GetApiValue(8, FORMAT_OCT, 42);
  Api.ApiStore(Val);

  Val := GetApiValue(16, FORMAT_BIN, 42);
  Api.ApiStore(Val);

  Val := GetApiValue(3, 10, 42);
  Api.ApiStore(Val);

  writeln(Api.GetText);
  readln;
  Api.Free;
end.

